#!usr/bin/env python
# -*- coding: utf8 -*-
#
# -----------------------------------------------------------------------------
#   File: conversational_settings.py (as part of project URUMETRICS-CODE)
#   Created: 25/08/2022 10:34
#   Last Modified: 25/08/2022 10:34
# -----------------------------------------------------------------------------
#   Author: William N. Havard
#           Postdoctoral Researcher
#
#   Mail  : william.havard@ens.fr / william.havard@gmail.com
#  
#   Institution: ENS / Laboratoire de Sciences Cognitives et Psycholinguistique
#
# ------------------------------------------------------------------------------
#   Description: 
#       • 
# -----------------------------------------------------------------------------


from conversations import Conversation
from conversations.standards import standard_filtering_rules, standard_path_selection_rules, \
                                    standard_turn_transition_rules


# Define the settings used by the conversational analysis package
uru_conversations = Conversation(# Participants
                                target_participant='CHI',
                                interactants = ['FEM', 'MAL'],
                                # Connectivity
                                allowed_gap=1000,
                                allow_segment_jump=False,
                                # Arguments for the user-defined/default_* functions
                                allow_multi_unit_turns=True,
                                allow_interactions_btw_interactants=False,
                                selection_key=['num_turns'],
                                # User-defined rules
                                # Define how turns are allowed to take place
                                turn_transition_rules=standard_turn_transition_rules,
                                # Select one path amongst several
                                best_path_selection_rules=standard_path_selection_rules,
                                # Filter out interactional sequences
                                filtering_rules=standard_filtering_rules, )
